@@ -28,7 +28,7 @@ use Scalar::Util qw(blessed);
 
 # NOTE: due to CPAN version checks this cannot currently be changed to a
 # standard version string, i.e. '0.21'
-our $VERSION   = '40';
+our $VERSION   = '47';
 our $pre_link  = undef;
 our $fast_mode = undef;
 our %nagios_setup;
@@ -44,9 +44,14 @@ sub NAGIOS_V2_ONLY    { 1 << 6 }    # not valid for nagios v1
 sub NAGIOS_NO_DISPLAY { 1 << 7 }    # should not be displayed by gui
 sub NAGIOS_V3         { 1 << 8 }    # nagios v3 attribute
 sub NAGIOS_V3_ONLY    { 1 << 9 }    # not valid for nagios v1 or v2
+sub NAGIOS_GROUP_SYNC { 1 << 10 }   # keep sync'ed with members method in group object
 
 # export constants - the :all tag will export them all
-our %EXPORT_TAGS = ( all => [ qw(NAGIOS_NO_INHERIT NAGIOS_PERL_ONLY NAGIOS_V1 NAGIOS_V2 NAGIOS_V3 NAGIOS_V1_ONLY NAGIOS_V2_ONLY NAGIOS_V3_ONLY NAGIOS_NO_DISPLAY) ] );
+our %EXPORT_TAGS = (
+    all => [
+        qw(NAGIOS_NO_INHERIT NAGIOS_PERL_ONLY NAGIOS_V1 NAGIOS_V2 NAGIOS_V3 NAGIOS_V1_ONLY NAGIOS_V2_ONLY NAGIOS_V3_ONLY NAGIOS_NO_DISPLAY NAGIOS_GROUP_SYNC)
+    ]
+);
 Exporter::export_ok_tags('all');
 
 # we also export %nagios_setup only if it is asked for by name
@@ -116,6 +121,7 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
         servicegroup_name => [ 'STRING',               18 ],
         alias             => [ 'STRING',               16 ],
         members => [ [ 'Nagios::Host', 'Nagios::Service' ], 16 ],
+        servicegroup_members => [ ['Nagios::ServiceGroup'], 280 ],
         name    => [ 'servicegroup_name', 22 ],
         comment => [ 'comment',           22 ],
         file    => [ 'filename',          22 ]
@@ -126,7 +132,7 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
         alias     => [ 'STRING',       280 ],
         address   => [ 'STRING',       280 ],
         parents    => [ ['Nagios::Host'],      280 ],
-        hostgroups => [ ['Nagios::HostGroup'], 280 ],
+        hostgroups => [ ['Nagios::HostGroup'], 1304 ],
         check_command      => [ 'STRING',  280 ],
         max_check_attempts => [ 'INTEGER', 280 ],
         checks_enabled     => [ 'BINARY',  280 ],
@@ -148,7 +154,7 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
         process_perf_data            => [ 'BINARY', 280 ],
         retain_status_information    => [ 'BINARY', 280 ],
         retain_nonstatus_information => [ 'BINARY', 280 ],
-        notification_period => [ ['Nagios::TimePeriod'], 280 ],
+        notification_period => [ 'Nagios::TimePeriod', 280 ],
         notification_interval => [ 'INTEGER', 280 ],
         notification_options => [ [qw(d u r)], 280 ],
         notifications_enabled => [ 'BINARY', 280 ],
@@ -169,8 +175,9 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
         hostgroup_name => [ 'STRING',            280 ],
         alias          => [ 'STRING',            280 ],
         contact_groups => [ ['Nagios::ContactGroup'], 40 ],
-        members        => [ ['Nagios::Host'],         280 ],
-        name    => [ 'hostgroup', 280 ],
+        members        => [ ['Nagios::Host'],         1304 ],
+        hostgroup_members => [ ['Nagios::HostGroup'], 280 ],
+        name    => [ 'hostgroup_name', 280 ],
         comment => [ 'comment',   280 ],
         file    => [ 'filename',  280 ]
     },
@@ -197,7 +204,7 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
         address4                      => [ 'STRING', 16 ],
         address5                      => [ 'STRING', 16 ],
         address6                      => [ 'STRING', 16 ],
-        contactgroups => [ ['Nagios::ContactGroup'], 16 ],
+        contactgroups => [ ['Nagios::ContactGroup'], 1040 ],
         name    => [ 'contact_name', 280 ],
         comment => [ 'comment',      280 ],
         file    => [ 'filename',     280 ]
@@ -206,7 +213,8 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
         use               => [ 'Nagios::ContactGroup', 280 ],
         contactgroup_name => [ 'STRING',               280 ],
         alias             => [ 'STRING',               280 ],
-        members => [ ['Nagios::Contact'], 280 ],
+        members => [ ['Nagios::Contact'], 1304 ],
+        contactgroup_members => [ ['Nagios::ContactGroup'], 280 ],
         name    => [ 'contactgroup_name', 280 ],
         comment => [ 'comment',           280 ],
         file    => [ 'filename',          280 ]
@@ -248,7 +256,7 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
     },
     ServiceEscalation => {
         use       => [ 'Nagios::ServiceEscalation', 280 ],
-        host_name => [ 'Nagios::Host',              280 ],
+        host_name => [ ['Nagios::Host'],            280 ],
         hostgroup_name => [ ['Nagios::HostGroup'], 280 ],
         service_description => [ 'Nagios::Service', 280 ],
         contacts       => [ ['Nagios::Contact'],      280 ],
@@ -264,11 +272,11 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
     },
     ServiceDependency => {
         use                           => [ 'Nagios::ServiceDependency', 280 ],
-        dependent_host_name           => [ 'Nagios::Host',              280 ],
+        dependent_host_name           => [ ['Nagios::Host'],            280 ],
         dependent_service_description => [ 'Nagios::Service',           280 ],
-        hostgroup_name                => [ 'Nagios::HostGroup',         280 ],
-        dependent_hostgroup_name      => [ 'Nagios::HostGroup',         280 ],
-        host_name                     => [ 'Nagios::Host',              280 ],
+        hostgroup_name                => [ ['Nagios::HostGroup'],       280 ],
+        dependent_hostgroup_name      => [ ['Nagios::HostGroup'],       280 ],
+        host_name                     => [ ['Nagios::Host'],            280 ],
         service_description           => [ 'Nagios::Service',           280 ],
         inherits_parent               => [ 'INTEGER',                   280 ],
         execution_failure_criteria    => [ [qw(o w u c n)], 280 ],
@@ -281,8 +289,8 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
     },
     HostEscalation => {
         use       => [ 'Nagios::HostEscalation', 280 ],
-        host_name => [ 'Nagios::Host',           280 ],
-        hostgroup => [ 'Nagios::HostGroup',      280 ],
+        host_name => [ ['Nagios::Host'],         280 ],
+        hostgroup => [ ['Nagios::HostGroup'],    280 ],
         contacts       => [ ['Nagios::Contact'],      280 ],
         contact_groups => [ ['Nagios::ContactGroup'], 280 ],
         first_notification    => [ 'INTEGER',   280 ],
@@ -295,9 +303,10 @@ push( @Nagios::Object::EXPORT_OK, '%nagios_setup' );
     },
     HostDependency => {
         use                      => [ 'Nagios::HostDependency', 280 ],
-        dependent_host_name      => [ 'Nagios::Host',           280 ],
-        dependent_hostgroup_name => [ 'Nagios::HostGroup',      280 ],
-        host_name                => [ 'Nagios::Host',           280 ],
+        dependent_host_name      => [ ['Nagios::Host'],         280 ],
+        dependent_hostgroup_name => [ ['Nagios::HostGroup'],    280 ],
+        host_name                => [ ['Nagios::Host'],         280 ],
+        hostgroup_name           => [ ['Nagios::HostGroup'],    280 ],
         inherits_parent          => [ 'INTEGER',                16 ],
         notification_failure_criteria => [ [qw(o w u c n)], 280 ],
         notification_failure_options  => [ [qw(o w u c n)], 280 ],
@@ -460,11 +469,13 @@ sub new {
 
     #print "type: $type, key: $nagios_setup_key\n";
 
-    confess "invalid type '$type' for Nagios::Object - does not exist in \%nagios_setup"
+    confess
+        "invalid type '$type' for Nagios::Object - does not exist in \%nagios_setup"
         if ( !exists $nagios_setup{$nagios_setup_key} );
 
     # set everything to undef by default
-    my %default = map { $_ => undef } keys %{ $nagios_setup{$nagios_setup_key} };
+    my %default
+        = map { $_ => undef } keys %{ $nagios_setup{$nagios_setup_key} };
 
     # if pre_link is set, don't set objects' resolved/registered flag
     if ($pre_link) {
@@ -491,7 +502,8 @@ sub new {
             # timeranges must be parsed into ARRAYs, so parse it here so that
             # users don't have to figure out the arrays and so we don't have
             # to export parse_time_range
-            if ( $nagios_setup{$nagios_setup_key}->{$key}[0] eq 'TIMERANGE' ) {
+            if ( $nagios_setup{$nagios_setup_key}->{$key}[0] eq 'TIMERANGE' )
+            {
                 $args{$key} = parse_time_range( $args{$key} );
             }
             $default{$key} = $args{$key};
@@ -626,7 +638,8 @@ sub template {
             return $self->{_use};
         }
         else {
-            confess "Unable to walk object heirarchy without object configuration.";
+            confess
+                "Unable to walk object heirarchy without object configuration.";
         }
     }
 }
@@ -649,18 +662,20 @@ Which is just short for:
 my $_name_hack;
 
 sub name {
-    my $self        = shift;
-    my $name_method = $self->_name_attribute;
-
-    if ( $name_method eq 'generated' ) {
-        $_name_hack++;
-        return ref($self) . '-' . $_name_hack;    # FIXME: this should work but feels wrong
-    }
+    my $self = shift;
 
     if ( !$self->register ) {
         return $self->{name};
     }
     else {
+        my $name_method = $self->_name_attribute;
+        if ( $name_method eq 'generated' ) {
+            $_name_hack++;
+            return
+                ref($self) . '-'
+                . $_name_hack;    # FIXME: this should work but feels wrong
+        }
+
         my $name = $self->$name_method();
 
         # recurse down on references to get the names, then generate something
@@ -836,11 +851,11 @@ sub _set ($ $ $) {
 
     if ( !$pre_link && !$fast_mode && exists $vf->{$key} ) {
 
-        # validate passed in arugments against arrayref in $vf (\%valid_fields)
+       # validate passed in arugments against arrayref in $vf (\%valid_fields)
         $self->_validate( $key, $value, @{ $vf->{$key} } );
     }
 
-    if ( ref $vf->{$key}[0] eq 'ARRAY' && $value =~ /,/ ) {
+    if ( ref $vf->{$key}[0] eq 'ARRAY' && defined($value) && $value =~ /,/ ) {
         $value = [ split /\s*,\s*/, $value ];
     }
 
@@ -875,7 +890,9 @@ sub _validate {
             # process single values as an arrayref anyways for consistency
             if ( ref($value) ne 'ARRAY' ) { $value = [$value] }
             foreach my $val (@$value) {
-                croak "object isa '" . ref($val) . "' when it should be a '$type'"
+                croak "object isa '"
+                    . ref($val)
+                    . "' when it should be a '$type'"
                     if ( ref($val) ne $type->[0] );
             }
         }
@@ -951,7 +968,8 @@ sub set_hostgroup_name {
         return $self->set_hostgroup( [ @existing, shift ] );
     }
     else {
-        confess "Called set_hostgroup() on an object that doesn't support it.";
+        confess
+            "Called set_hostgroup() on an object that doesn't support it.";
     }
 }
 
@@ -1010,7 +1028,7 @@ GENESIS: {
             # name() is a special case and is implemented by hand
             next if ( $method eq 'name' );
 
-            # the members() method in ServiceGroup is implemented manually (below)
+        # the members() method in ServiceGroup is implemented manually (below)
             next
                 if ( $pkg eq 'Nagios::ServiceGroup' && $method eq 'members' );
 
@@ -1065,7 +1083,8 @@ sub AUTOLOAD {
         $pkg->_make_method($setup_field);
     }
     else {
-        confess "Invalid method call.   $pkg does not know about method $method.";
+        confess
+            "Invalid method call.   $pkg does not know about method $method.";
     }
 
     goto \&{$AUTOLOAD};
@@ -1119,7 +1138,8 @@ sub set_members {
     # has resolved yet
     if ( $self->resolved ) {
         foreach my $item (@_) {
-            confess "set_members() arguments must be objects after resolve_objects() has been called."
+            confess
+                "set_members() arguments must be objects after resolve_objects() has been called."
                 unless ( ref($item) );
             push @members, $item;
         }
