@@ -25,7 +25,7 @@ use Symbol;
 
 # NOTE: due to CPAN version checks this cannot currently be changed to a
 # standard version string, i.e. '0.21'
-our $VERSION = '41';
+our $VERSION = '44';
 
 # this is going to be rewritten to use AUTOLOAD + method caching in a future version
 BEGIN {
@@ -457,7 +457,8 @@ sub update_v3 ($) {
     open( $log_fh, "<$self->{LOGFILE}" )
         || croak "could not open file $self->{LOGFILE} for reading: $!";
 
-    my %valid_types = map { ( $_ => 1 ) } qw(info programstatus hoststatus servicestatus contactstatus servicecomment hostcomment servicedowntime hostdowntime);
+    my %valid_types = map { ( $_ => 1 ) }
+        qw(info programstatus hoststatus servicestatus contactstatus servicecomment hostcomment servicedowntime hostdowntime);
     my $entry = '';
     my %attributes;
     my $type = 0;
@@ -465,13 +466,15 @@ sub update_v3 ($) {
         next if ( $line =~ /^\s*#|^\s*$/ );
         if ( $line =~ /^\s*(\w+)\s*{\s*$/ ) {
             %attributes = ();
-            if (exists $valid_types{$1}) {
+            if ( exists $valid_types{$1} ) {
                 $type = $1;
-            } else {
+            }
+            else {
                 $type = 0;
             }
         }
         if ( $line =~ /^\s*}\s*$/ ) {
+
             # Only save the object if it is a valid type
             if ($type) {
                 $handlers{$type}->( \%attributes );
@@ -537,16 +540,16 @@ Nagios::Service::Status has the following accessor methods (For V1):
 sub service {
     my ( $self, $host, $service ) = @_;
 
-    if ( ref $host eq 'Nagios::Host' ) {
+    if ( ref($host) eq 'Nagios::Host' ) {
         $host = $host->host_name;
     }
 
     # allow just a service to be passed in
-    if ( ref $host eq 'Nagios::Service' ) {
+    if ( ref($host) eq 'Nagios::Service' ) {
         $service = $host;
         $host    = $service->host_name;
     }
-    if ( ref $service eq 'Nagios::Service' ) {
+    if ( ref($service) eq 'Nagios::Service' ) {
         $service = $service->service_description;
     }
 
@@ -590,7 +593,7 @@ Returns an array of services descriptions for a given host.
 
 sub list_services_on_host {
     my ( $self, $host ) = @_;
-    if ( ref $host && UNIVERSAL::can( $host, 'host_name' ) ) {
+    if ( ref($host) && UNIVERSAL::can( $host, 'host_name' ) ) {
         $host = $host->host_name;
     }
     return keys %{ $self->{SERVICE}{$host} };
@@ -631,7 +634,7 @@ Nagios::Host::Status has the following accessor methods (for V1):
 sub host {
     my ( $self, $host ) = @_;
 
-    if ( ref $host =~ /^Nagios::(Host|Service)$/ ) {
+    if ( ref($host) =~ /^Nagios::(Host|Service)$/ ) {
         $host = $host->host_name;
     }
 
@@ -652,6 +655,16 @@ Returns a simple array of host names (no objects).
 
 sub list_hosts { keys %{ $_[0]->{HOST} }; }
 
+=item list_hostdowntime()
+
+Returns a simple array of host downtimes (no objects)
+
+ my @hostdowntimes = $log->list_hostdowntime;
+
+=cut
+
+sub list_hostdowntime { keys %{ $_[0]->{HOSTDOWNTIME} }; }
+
 =item info() [Nagios v2 & v3 logs only]
 
 Returns a Nagios::Info::Status object.   It only has two methods, created()
@@ -693,7 +706,7 @@ Nagios::Contact::Status has the following accessor methods (for v3):
 sub contact {
     my ( $self, $name ) = @_;
 
-    $name = $name->contact_name if ( ref $name eq 'Nagios::Contact' );
+    $name = $name->contact_name if ( ref($name) eq 'Nagios::Contact' );
 
     return undef if ( !$self->{CONTACT}{$name} );
 
@@ -729,7 +742,7 @@ Nagios::Hostcomment::Status is a perl HASH, keyed with the Nagios comment IDs, w
 sub hostcomment {
     my ( $self, $host ) = @_;
 
-    $host = $host->host_name if ( ref $host =~ /^Nagios::(Host|Service)$/ );
+    $host = $host->host_name if ( ref($host) =~ /^Nagios::(Host|Service)$/ );
 
     return undef if ( !$self->{HOSTCOMMENT}{$host} );
     foreach my $id ( keys %{ $self->{HOSTCOMMENT}{$host} } ) {
@@ -770,17 +783,20 @@ Nagios::Servicecomment::Status is a perl HASH, keyed with the Nagios comment IDs
 sub servicecomment {
     my ( $self, $host, $service ) = @_;
 
-    $host = $host->host_name if ( ref $host eq 'Nagios::Host' );
+    $host = $host->host_name if ( ref($host) eq 'Nagios::Host' );
 
     # allow just a service to be passed in
-    if ( ref $host eq 'Nagios::Service' ) {
+    if ( ref($host) eq 'Nagios::Service' ) {
         $service = $host;
         $host    = $service->host_name;
     }
     $service = $service->service_description
-        if ( ref $service eq 'Nagios::Service' );
+        if ( ref($service) eq 'Nagios::Service' );
 
-    return undef if ( !$self->{SERVICECOMMENT}{$host}{$service} );
+    return undef
+        if ( !$host
+        || !$service
+        || !$self->{SERVICECOMMENT}{$host}{$service} );
     foreach my $id ( keys %{ $self->{SERVICECOMMENT}{$host}{$service} } ) {
         bless( $self->{SERVICECOMMENT}{$host}{$service}{$id},
             'Nagios::Servicecomment::Status' );
@@ -818,7 +834,7 @@ Nagios::Hostdowntime::Status is a perl HASH, keyed with the Nagios downtime IDs,
 sub hostdowntime {
     my ( $self, $host ) = @_;
 
-    $host = $host->host_name if ( ref $host =~ /^Nagios::(Host|Service)$/ );
+    $host = $host->host_name if ( ref($host) =~ /^Nagios::(Host|Service)$/ );
 
     return undef if ( !$self->{HOSTDOWNTIME}{$host} );
     foreach my $id ( keys %{ $self->{HOSTDOWNTIME}{$host} } ) {
@@ -862,15 +878,15 @@ Nagios::Servicedowntime::Status is a perl HASH, keyed with the Nagios downtime I
 sub servicedowntime {
     my ( $self, $host, $service ) = @_;
 
-    $host = $host->host_name if ( ref $host eq 'Nagios::Host' );
+    $host = $host->host_name if ( ref($host) eq 'Nagios::Host' );
 
     # allow just a service to be passed in
-    if ( ref $host eq 'Nagios::Service' ) {
+    if ( ref($host) eq 'Nagios::Service' ) {
         $service = $host;
         $host    = $service->host_name;
     }
     $service = $service->service_description
-        if ( ref $service eq 'Nagios::Service' );
+        if ( ref($service) eq 'Nagios::Service' );
 
     return undef if ( !$self->{SERVICEDOWNTIME}{$host}{$service} );
     foreach my $id ( keys %{ $self->{SERVICEDOWNTIME}{$host}{$service} } ) {
